@@ -24,6 +24,7 @@
 #include <zim/entry.h>
 #include <zim/item.h>
 #include <zim/error.h>
+#include <zim/tools.h>
 #include "fileimpl.h"
 #include "tools.h"
 #include "log.h"
@@ -41,8 +42,16 @@ namespace zim
     : m_impl(new FileImpl(fd))
     { }
 
+  Archive::Archive(FdInput fd)
+    : m_impl(new FileImpl(fd))
+    { }
+
   Archive::Archive(int fd, offset_type offset, size_type size)
-    : m_impl(new FileImpl(fd, offset_t(offset), zsize_t(size)))
+    : Archive(FdInput(fd, offset, size))
+  {}
+
+  Archive::Archive(const std::vector<FdInput>& fds)
+    : m_impl(new FileImpl(fds))
     { }
 #endif
 
@@ -85,14 +94,19 @@ namespace zim
 
   entry_index_type Archive::getMediaCount() const
   {
-    return countMimeType(
-      getMetadata("Counter"),
-      [](const std::string& mimetype) {
-        return mimetype.find("image/") == 0 ||
-               mimetype.find("video/") == 0 ||
-               mimetype.find("audio/") == 0;
-      }
-    );
+    try {
+      return countMimeType(
+        getMetadata("Counter"),
+        [](const std::string& mimetype) {
+          return mimetype.find("image/") == 0 ||
+                 mimetype.find("video/") == 0 ||
+                 mimetype.find("audio/") == 0;
+        }
+      );
+    } catch(const EntryNotFound& e) {
+      return (m_impl->getNamespaceEntryCount('I').v
+            + m_impl->getNamespaceEntryCount('J').v);
+    }
   }
 
   Uuid Archive::getUuid() const
@@ -122,7 +136,7 @@ namespace zim
     auto end = m_impl->getNamespaceEndOffset('M');
     for (auto idx=start; idx!=end; idx++) {
       auto dirent = m_impl->getDirent(idx);
-      ret.push_back(dirent->getUrl());
+      ret.push_back(dirent->getPath());
     }
     return ret;
   }
@@ -141,9 +155,8 @@ namespace zim
   }
 
   Item Archive::getIllustrationItem(unsigned int size) const {
-    std::ostringstream ss;
-    ss  << "Illustration_" << size << "x" << size << "@" << 1;
-    auto r = m_impl->findx('M', ss.str());
+    auto r = m_impl->findx('M', Formatter() << "Illustration_" << size << "x"
+                                            << size << "@" << 1);
     if (r.first) {
       return getEntryByPath(entry_index_type(r.second)).getItem();
     }
@@ -245,6 +258,15 @@ namespace zim
     throw EntryNotFound("Cannot find entry");
   }
 
+  Entry Archive::getEntryByPathWithNamespace(char ns, const std::string& path) const
+  {
+    auto r = m_impl->findx(ns, path);
+    if (r.first) {
+      return Entry(m_impl, entry_index_type(r.second));
+    }
+    throw EntryNotFound("Cannot find entry");
+  }
+
   Entry Archive::getEntryByTitle(entry_index_type idx) const
   {
     return Entry(m_impl, entry_index_type(m_impl->getIndexByTitle(title_index_t(idx))));
