@@ -21,7 +21,9 @@
 #include <zim/zim.h>
 #include <zim/archive.h>
 #include <zim/item.h>
+#if defined(ENABLE_XAPIAN)
 #include <zim/search.h>
+#endif
 #include <zim/suggestion.h>
 #include <zim/error.h>
 
@@ -105,6 +107,9 @@ TEST(ZimArchive, openingAnEmptyZimArchiveSucceeds)
 
   zim::Archive archive(tmpfile->path());
   ASSERT_TRUE(archive.check());
+
+  ASSERT_EQ(archive.getMediaCount(), 0U);
+  ASSERT_EQ(archive.getArticleCount(), 0U);
 }
 
 bool isNastyOffset(int offset) {
@@ -171,15 +176,15 @@ TEST(ZimArchive, openCreatedArchive)
   zim::Archive archive(tempPath);
 #if !defined(ENABLE_XAPIAN)
 // 2*listingIndex + M/Counter + M/Title + mainpage + 2*Illustration + 2*Item + redirection
-#define ALL_ENTRY_COUNT 10
+#define ALL_ENTRY_COUNT 10U
 #else
 // same as above + 2 xapian indexes.
-#define ALL_ENTRY_COUNT 12
+#define ALL_ENTRY_COUNT 12U
 #endif
   ASSERT_EQ(archive.getAllEntryCount(), ALL_ENTRY_COUNT);
 #undef ALL_ENTRY_COUNT
-  ASSERT_EQ(archive.getEntryCount(), 3);
-  ASSERT_EQ(archive.getArticleCount(), 1);
+  ASSERT_EQ(archive.getEntryCount(), 3U);
+  ASSERT_EQ(archive.getArticleCount(), 1U);
   ASSERT_EQ(archive.getUuid(), uuid);
   ASSERT_EQ(archive.getMetadataKeys(), std::vector<std::string>({"Counter", "Illustration_48x48@1", "Illustration_96x96@1", "Title"}));
   ASSERT_EQ(archive.getIllustrationSizes(), std::set<unsigned int>({48, 96}));
@@ -189,7 +194,12 @@ TEST(ZimArchive, openCreatedArchive)
   auto titleMeta = archive.getMetadataItem("Title");
   ASSERT_EQ(std::string(titleMeta.getData()), "This is a title");
   ASSERT_EQ(titleMeta.getMimetype(), "text/plain;charset=utf-8");
+
+  auto titleMeta_with_ns = archive.getEntryByPathWithNamespace('M', "Title");
+  ASSERT_EQ(titleMeta.getIndex(), titleMeta_with_ns.getIndex());
+
   ASSERT_EQ(archive.getMetadata("Counter"), "text/html=2");
+
   auto illu48 = archive.getIllustrationItem(48);
   ASSERT_EQ(illu48.getPath(), "Illustration_48x48@1");
   ASSERT_EQ(std::string(illu48.getData()), "PNGBinaryContent48");
@@ -207,6 +217,9 @@ TEST(ZimArchive, openCreatedArchive)
   ASSERT_THROW(foo.getRedirectEntry(), zim::InvalidType);
   ASSERT_THROW(foo.getRedirectEntryIndex(), zim::InvalidType);
 
+  auto foo_with_ns = archive.getEntryByPathWithNamespace('C', "foo");
+  ASSERT_EQ(foo.getIndex(), foo_with_ns.getIndex());
+
   auto foo2 = archive.getEntryByPath("foo2");
   ASSERT_EQ(foo2.getPath(), "foo2");
   ASSERT_EQ(foo2.getTitle(), "AFoo");
@@ -224,6 +237,11 @@ TEST(ZimArchive, openCreatedArchive)
   ASSERT_EQ(main.getRedirectEntry().getIndex(), foo.getIndex());
   ASSERT_EQ(main.getRedirectEntryIndex(), foo.getIndex());
   ASSERT_EQ(archive.getMainEntryIndex(), main.getIndex());
+
+  // NO existant entries
+  ASSERT_THROW(archive.getEntryByPath("non/existant/path"), zim::EntryNotFound);
+  ASSERT_THROW(archive.getEntryByPath("C/non/existant/path"), zim::EntryNotFound);
+  ASSERT_THROW(archive.getEntryByPathWithNamespace('C', "non/existant/path"), zim::EntryNotFound);
 }
 
 #if WITH_TEST_DATA
@@ -248,6 +266,62 @@ TEST(ZimArchive, openRealZimArchive)
   }
 }
 
+TEST(ZimArchive, openSplitZimArchive)
+{
+  const char* fname = "wikibooks_be_all_nopic_2017-02_splitted.zim";
+
+  for (auto& testfile: getDataFilePath(fname)) {
+    const TestContext ctx{ {"path", testfile.path+"aa" } };
+    std::unique_ptr<zim::Archive> archive;
+    EXPECT_NO_THROW( archive.reset(new zim::Archive(testfile.path+"aa")) ) << ctx;
+    if ( archive ) {
+      EXPECT_TRUE( archive->check() ) << ctx;
+    }
+  }
+}
+
+TEST(ZimArchive, openDontFallbackOnNonSplitZimArchive)
+{
+  const char* fname = "wikibooks_be_all_nopic_2017-02.zim";
+
+  for (auto& testfile: getDataFilePath(fname)) {
+    const TestContext ctx{ {"path", testfile.path+"aa" } };
+    try {
+      zim::Archive(testfile.path+"aa");
+      FAIL(); // Exception is expected
+    } catch(const std::runtime_error& e) {
+      const std::string expected = std::string("Error opening as a split ZIM file: ") + testfile.path + "aa";
+      EXPECT_EQ(expected, e.what()) << ctx;
+    }
+  }
+}
+
+TEST(ZimArchive, openNonExistantZimArchive)
+{
+  const std::string fname = "non_existant.zim";
+
+  try {
+    zim::Archive archive(fname);
+    FAIL(); // Exception is expected
+  } catch(const std::runtime_error& e) {
+    const std::string expected = std::string("Error opening ZIM file: ") + fname;
+    EXPECT_EQ(expected, e.what()) << fname;
+  }
+}
+
+TEST(ZimArchive, openNonExistantZimSplitArchive)
+{
+  const std::string fname = "non_existant.zimaa";
+
+  try {
+    zim::Archive archive(fname);
+    FAIL(); // Exception is expected
+  } catch(const std::runtime_error& e) {
+    const std::string expected = std::string("Error opening as a split ZIM file: ") + fname;
+    EXPECT_EQ(expected, e.what()) << fname;
+  }
+}
+
 TEST(ZimArchive, randomEntry)
 {
   const char* const zimfiles[] = {
@@ -418,7 +492,7 @@ TEST(ZimArchive, validate)
 
   TEST_BROKEN_ZIM_NAME(
     "invalid.invalid_checksumpos.zim",
-    "Checksum position is not valid\n"
+    "Zim file(s) is of bad size or corrupted.\n"
   );
 
   TEST_BROKEN_ZIM_NAME(
@@ -446,6 +520,11 @@ TEST(ZimArchive, validate)
     "Invalid cluster pointer\n"
   );
 
+  TEST_BROKEN_ZIM_NAME(
+    "invalid.offset_in_cluster.zim",
+     "Error parsing cluster. Offsets are not ordered.\n"
+  )
+
 
   for(auto& testfile: getDataFilePath("invalid.nonsorted_dirent_table.zim")) {
     std::string expected;
@@ -481,7 +560,7 @@ TEST(ZimArchive, validate)
     EXPECT_BROKEN_ZIMFILE(testfile.path, expected)
   }
 }
-#endif
+
 
 void checkEquivalence(const zim::Archive& archive1, const zim::Archive& archive2)
 {
@@ -492,7 +571,7 @@ void checkEquivalence(const zim::Archive& archive1, const zim::Archive& archive2
   const zim::Entry mainEntry = archive1.getMainEntry();
   ASSERT_EQ(mainEntry.getTitle(), archive2.getMainEntry().getTitle());
 
-  ASSERT_NE(0, archive1.getEntryCount()); // ==> below loop is not a noop
+  ASSERT_NE(0U, archive1.getEntryCount()); // ==> below loop is not a noop
   {
     auto range1 = archive1.iterEfficient();
     auto range2 = archive2.iterEfficient();
@@ -559,7 +638,6 @@ void checkEquivalence(const zim::Archive& archive1, const zim::Archive& archive2
 #endif
 }
 
-#if WITH_TEST_DATA
 TEST(ZimArchive, multipart)
 {
   auto nonSplittedZims = getDataFilePath("wikibooks_be_all_nopic_2017-02.zim");
@@ -609,7 +687,39 @@ TEST(ZimArchive, openZIMFileEmbeddedInAnotherFile)
   for(auto i=0UL; i < normalZims.size(); i++) {
     const zim::Archive archive1(normalZims[i].path);
     const int fd = OPEN_READ_ONLY(embeddedZims[i].path);
-    const zim::Archive archive2(fd, 8, archive1.getFilesize());
+    const zim::Archive archive2(zim::FdInput(fd, 8, archive1.getFilesize()));
+    close(fd);
+
+    checkEquivalence(archive1, archive2);
+  }
+}
+
+TEST(ZimArchive, openZIMFileMultiPartEmbeddedInAnotherFile)
+{
+  auto normalZims = getDataFilePath("small.zim");
+  auto embeddedZims = getDataFilePath("small.zim.embedded.multi");
+
+  ASSERT_EQ(normalZims.size(), embeddedZims.size()) << "We must have same number of zim files. (This is a test data issue)";
+  for(auto i=0UL; i < normalZims.size(); i++) {
+    const zim::Archive archive1(normalZims[i].path);
+    auto archive_size = archive1.getFilesize();
+
+    std::vector<zim::FdInput> fds;
+    zim::offset_type start_offset = std::string("BEGINZIMMULTIPART").size();
+    while (archive_size > 2048) {
+      int fd = OPEN_READ_ONLY(embeddedZims[i].path);
+      fds.push_back(zim::FdInput(fd, start_offset, 2048));
+      start_offset += 2048 + std::string("NEWSECTIONZIMMULTI").size();
+      archive_size -= 2048;
+    }
+    int fd = OPEN_READ_ONLY(embeddedZims[i].path);
+    fds.push_back(zim::FdInput(fd, start_offset, archive_size));
+
+    const zim::Archive archive2(fds);
+
+    for(auto &fd: fds) {
+      close(fd.fd);
+    }
 
     checkEquivalence(archive1, archive2);
   }
@@ -617,15 +727,16 @@ TEST(ZimArchive, openZIMFileEmbeddedInAnotherFile)
 #endif // not _WIN32
 #endif // WITH_TEST_DATA
 
+
+#if WITH_TEST_DATA
 zim::Blob readItemData(const zim::Item::DirectAccessInfo& dai, zim::size_type size)
 {
   zim::DEFAULTFS::FD fd(zim::DEFAULTFS::openFile(dai.first));
-  std::shared_ptr<char> data(new char[size]);
+  std::shared_ptr<char> data(new char[size], std::default_delete<char[]>());
   fd.readAt(data.get(), zim::zsize_t(size), zim::offset_t(dai.second));
   return zim::Blob(data, size);
 }
 
-#if WITH_TEST_DATA
 TEST(ZimArchive, getDirectAccessInformation)
 {
   for(auto& testfile:getDataFilePath("small.zim")) {
@@ -642,7 +753,7 @@ TEST(ZimArchive, getDirectAccessInformation)
         }
       }
     }
-    ASSERT_NE(0, checkedItemCount);
+    ASSERT_NE(0U, checkedItemCount);
   }
 }
 
@@ -664,7 +775,7 @@ TEST(ZimArchive, getDirectAccessInformationInAnArchiveOpenedByFD)
         }
       }
     }
-    ASSERT_NE(0, checkedItemCount);
+    ASSERT_NE(0U, checkedItemCount);
   }
 }
 
@@ -677,7 +788,7 @@ TEST(ZimArchive, getDirectAccessInformationFromEmbeddedArchive)
   for(auto i=0UL; i < normalZims.size(); i++) {
     const int fd = OPEN_READ_ONLY(embeddedZims[i].path);
     const auto size = zim::DEFAULTFS::openFile(normalZims[i].path).getSize();
-    const zim::Archive archive(fd, 8, size.v);
+    const zim::Archive archive(zim::FdInput(fd, 8, size.v));
     zim::entry_index_type checkedItemCount = 0;
     for ( auto entry : archive.iterEfficient() ) {
       if (!entry.isRedirect()) {
@@ -690,7 +801,7 @@ TEST(ZimArchive, getDirectAccessInformationFromEmbeddedArchive)
         }
       }
     }
-    ASSERT_NE(0, checkedItemCount);
+    ASSERT_NE(0U, checkedItemCount);
   }
 }
 #endif // not _WIN32
